package com.chelvc.cloud.vehicle.client;


import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.CommentDTO;
import com.chelvc.cloud.vehicle.client.param.AddGoodsCommentParam;
import com.chelvc.cloud.vehicle.client.param.CommentQueryParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 商家评论业务接口
 *
 * @author xp
 * @data 2023/3/31
 */
@FeignClient("vehicle")
public interface CommentClient {
    /**
     * 获取商品评价列表
     *
     * @param goodsId    商品ID
     * @param parentId   父级评论id
     * @param merchantId 商户id
     * @param param      查询参数
     * @return 评价列表
     */
    @GetMapping("/comment/listGoodsComments")
    List<CommentDTO> listGoodsComments(@RequestParam("goodsId") Long goodsId, @RequestParam("parentId") Long parentId,
                                       @RequestParam("merchantId") Long merchantId,
                                       @SpringQueryMap CommentQueryParam param);

    /**
     * 新增商品评价
     *
     * @param param  新增参数
     * @param userId 用户标识
     */
    @PostMapping("/comment/addGoodsComments")
    void addGoodsComments(@RequestBody AddGoodsCommentParam param, @RequestParam("userId") Long userId);
}
