package com.chelvc.cloud.vehicle.client;

import com.chelvc.cloud.vehicle.client.constant.ExamineType;
import com.chelvc.cloud.vehicle.client.dto.DynamicContentDTO;
import com.chelvc.cloud.vehicle.client.param.AddDynamicContentParam;
import com.chelvc.cloud.vehicle.client.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.client.param.QueryDynamicContentParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 动态信息Service接口
 *
 * @USER: igl
 * @date: 2023/8/8 15:13
 */
@FeignClient("vehicle")
public interface DynamicContentClient {

    /**
     * 查询动态信息
     */
    @GetMapping("/dynamicContent/queryById")
    DynamicContentDTO queryById(@RequestParam("id") Long id, @RequestParam("userId") Long userId);

    /**
     * 查询动态信息列表
     */
    @GetMapping("/dynamicContent/queryPageList")
    Pagination<DynamicContentDTO> queryPageList(@SpringQueryMap QueryDynamicContentParam param,
                                                @RequestParam("pageNum") Integer pageNum,
                                                @RequestParam("pageSize") Integer pageSize,
                                                @RequestParam("userId") Long userId);

    /**
     * 查询动态信息列表
     */
    @GetMapping("/dynamicContent/queryAdminPageList")
    Pagination<DynamicContentDTO> queryAdminPageList(@SpringQueryMap QueryAdminDynamicParam param);

    /**
     * 查询动态信息列表
     */
    @GetMapping("/dynamicContent/queryUserById")
    Pagination<DynamicContentDTO> queryUserById(@RequestParam("selfUserId") Long selfUserId,
                                                @RequestParam("userId") Long userId,
                                                @RequestParam("pageNum") Integer pageNum,
                                                @RequestParam("pageSize") Integer pageSize);

    /**
     * 修改动态信息
     */
    @PostMapping("/dynamicContent/insertByBo")
    Long insertByBo(@RequestBody AddDynamicContentParam dto, @RequestParam("userId") Long userId);

    /**
     * 校验并批量删除动态信息信息
     */
    @DeleteMapping("/dynamicContent")
    void deleteByIds(@RequestParam("id") Long id, @RequestParam("userId") Long userId);

    /**
     * 添加评论数
     *
     * @param dynamicId
     */
    @PostMapping("/dynamicContent/addComment")
    void addComment(@RequestParam("dynamicId") Long dynamicId);

    /**
     * 减少评论数
     *
     * @param dynamicId
     */
    @PostMapping("/dynamicContent/reduceComment")
    void reduceComment(@RequestParam("dynamicId") Long dynamicId, @RequestParam("num") int num);

    /**
     * 给评论添加点赞/踩
     *
     * @param dynamicStatus
     * @param contentId
     */
    @PostMapping("/dynamicContent/addNum")
    void addNum(@RequestParam("dynamicStatus") Integer dynamicStatus, @RequestParam("contentId") Long contentId);

    /**
     * 取消评论点赞/踩
     *
     * @param dynamicStatus
     * @param contentId
     */
    @PostMapping("/dynamicContent/reduceNum")
    void reduceNum(@RequestParam("dynamicStatus") Integer dynamicStatus, @RequestParam("contentId") Long contentId);

    /**
     * 屏蔽/恢复某条动态信息
     *
     * @param id 主键
     */
    @PostMapping("/dynamicContent/shieldById")
    void shieldById(@RequestParam("id") Long id, @RequestParam("type") Integer type);

    /**
     * 屏蔽/恢复某个用户所有动态信息
     *
     * @param userId 主键
     * @param type   状态 0-正常；1-平台屏蔽
     */
    @PostMapping("/dynamicContent/shieldByUserId")
    void shieldByUserId(@RequestParam("userId") Long userId, @RequestParam("type") Integer type);

    /**
     * 动态手动审核
     *
     * @param id
     * @param examine
     * @author igl
     * @date 2023/8/17 17:25
     */
    @PostMapping("/dynamicContent/examine")
    void examine(@RequestParam("id") Long id, @RequestParam("examine") ExamineType examine);

    /**
     * 自动审核
     *
     * @param id             动态标识
     * @param platformStatus 平台状态
     * @param examineResult  审核结果
     * @author igl
     * @date 2023/9/12 17:25
     */
    @PostMapping("/dynamicContent/autoAudit")
    void autoAudit(@RequestParam("id") Long id, @RequestParam("platformStatus") int platformStatus,
                   @RequestParam("examineResult") int examineResult);

    /**
     * 视频审核回调
     *
     * @param param
     * @author igl
     * @date 2023/9/12 16:55
     */
    @PostMapping("/dynamicContent/videoAuditCallback")
    void videoAuditCallback(@RequestBody Map<String, Object> param);
}
