package com.chelvc.cloud.vehicle.client;

import com.chelvc.cloud.vehicle.client.constant.SystemInformType;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

@FeignClient("vehicle")
public interface InformClient {

    /**
     * 获取用户的订单通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/inform/selectOmsOrderOperateHistoryListByUserId")
    Pagination<OmsOrderOperateHistoryDTO> selectOmsOrderOperateHistoryListByUserId(
            @RequestParam("userId") Long userId, @RequestParam("paging") Paging paging);

    /**
     * 将新系统通知标识未读改为已读
     */
    @GetMapping("/inform/changeSystemInformFlag")
    void changeSystemInformFlag(@RequestParam("type") SystemInformType type, @RequestParam("userId") Long userId);

    /**
     * 查看是否有系统新消息
     */
    @GetMapping("/inform/getNewInform")
    NewInformDTO getNewInform(@RequestParam("userId") Long userId);

    /**
     * (商家端)查看是否有系统新消息
     */
    @GetMapping("/inform/getMerchantNewInform")
    NewMerchantInformDTO getMerchantNewInform(@RequestParam("merchantId") Long merchantId);

    /**
     * (商家端)获取订单退款申请列表
     *
     * @param paging 分页信息
     * @return 用户的订单通知列表
     */
    @GetMapping("/inform/listOrderReturnApply")
    Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(@RequestParam("merchantId") Long merchantId,
                                                               @RequestParam("paging") Paging paging);

    /**
     * 获取用户的支付通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单通知列表
     */
    @GetMapping("/inform/selectOmsOrderPayHistoryListByUserId")
    Pagination<OmsOrderDTO> selectOmsOrderPayHistoryListByUserId(@RequestParam("userId") Long userId,
                                                                 @RequestParam("paging") Paging paging);

    /**
     * 取用户的点赞回复通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单通知列表
     */
    @GetMapping("/inform/selectEvaluateListByUserId")
    Pagination<EvaluateDTO> selectEvaluateListByUserId(@RequestParam("userId") Long userId,
                                                       @RequestParam("paging") Paging paging);

    /**
     * 取用户的评价通知列表
     *
     * @param paging 分页信息
     * @return 用户的评价通知列表
     */
    @GetMapping("/inform/selectCommentListByUserId")
    Pagination<GoodsAssessDto> selectCommentListByUserId(@RequestParam("userId") Long userId,
                                                         @RequestParam("paging") Paging paging);

    /**
     * 取用户的优惠券通知列表
     *
     * @param paging 分页信息
     * @return 用户的评价通知列表
     */
    @GetMapping("/inform/selectUserCouponListByUserId")
    Pagination<UserCouponDTO> selectUserCouponListByUserId(@RequestParam("userId") Long userId,
                                                           @RequestParam("paging") Paging paging);
}
