package com.chelvc.cloud.vehicle.server.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.chelvc.framework.common.model.File;
import com.chelvc.framework.database.entity.Entity;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * 商家认证数据模型
 *
 * @author liude
 * @data 2023/12/10
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class MerchantAuth implements Entity<Long> {
    /**
     * 主键
     */
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    private String businessLicenseId; // 营业执照编号
    private String businessLicenseNumber; // 营业执照证件号
    private Date businessLicenseExpirationDate; // 营业执照有效期
    private String idCardNumber; // 身份证号
    private Date idCardExpirationDateStart; // 身份证有效期开始时间
    private Date idCardExpirationDate; // 身份证有效期
    private String storeImage; // 店铺图
    private String legalRepresentativeName; // 法人姓名
    private Long applicant; // 申请人
    private String legalRepresentativeGender; // 法人性别
    private String mobileNumber; // 手机号
    private String account; // 账号
    private Date accountRegistrationDate; // 账号注册日期
    private String storeType; // 店铺类型
    private String storeName; // 店名
    private String storeAddress; // 店铺地址
    private String businessLicensePhoto; // 经营许可证照片
    private String businessLicenseNumberValid; // 经营许可证号
    private Date businessLicenseExpirationDateValid; // 经营许可证有效期
    private Date applicationTime; // 申请时间
    private String reviewStatus; // 审核状态
    private String reviewer; // 审核人
    private String recommendationUser; // 推荐人
    private Date reviewTime; // 审核时间
    private String accountName; // 开户名称
    private String bankCardNo; // 开户账号
    private String reservedPhoneNo; //银行预留手机号

    /**
     * 营业执照证件号
     */
    private String businessPhoto;
    /**
     * 身份证正面照片
     */
    private String idCardFrontPhoto;

    /**
     * 身份证反面照片
     */
    private String idCardBackPhoto;

    /**
     * 店铺内部环境图
     */
    private List<File> storeInnerImage;

    /**
     * 店铺详细地址
     */
    @Size(max = 255, message = "店铺详细地址长度不能大于255")
    private String storeAddressDetail;

    /**
     * 邮箱
     */
    @Size(max = 255, message = "邮箱长度不能大于255")
    private String email;
    /**
     * 审核意见
     */
    private String message;
}
