package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.client.constant.ReserveStatus;
import com.chelvc.cloud.vehicle.client.constant.TradeConstant;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.TransferNotifyParam;
import com.chelvc.cloud.vehicle.client.param.WithdrawNotifyParam;
import com.chelvc.cloud.vehicle.client.param.WithdrawParam;
import com.chelvc.cloud.vehicle.server.dao.AssetMapper;
import com.chelvc.cloud.vehicle.server.entity.Asset;
import com.chelvc.cloud.vehicle.server.entity.MerchantRelation;
import com.chelvc.cloud.vehicle.server.entity.OmsWithdraw;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;

@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired,@Lazy}))
public class AssetServiceImpl extends ServiceImpl<AssetMapper, Asset> implements AssetService {
    private final BalanceDetailService balanceDetailService;
    private final OmsOrderService orderService;
    private final MerchantService merchantService;
    private final MerchantRelationService merchantRelationService;
    private final OmsWithdrawService withdrawService;

    @Override
    public Asset queryAsset(Long userId, Integer type) {
        LambdaQueryWrapper<Asset> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Asset::getUserId, userId);
        wrapper.eq(Asset::getType, type);
        Asset asset = baseMapper.selectOne(wrapper);
        if (asset != null) {
            return asset;
        }
        asset = new Asset();
        asset.setFrozen(new BigDecimal("0"));
        asset.setType(type);
        asset.setTotal(new BigDecimal("0"));
        asset.setUpdateTime(LocalDateTime.now());
        asset.setUserId(userId);
        baseMapper.insert(asset);
        return asset;
    }

    @Override
    public BigDecimal updateAsset(Integer operateType, BigDecimal operateAmount, Long userId, Integer type, Integer amountType) {
        Asset asset = queryAsset(userId, type);
        if(amountType == 1){
            BigDecimal total = asset.getTotal();
            if (operateType == 1 && total.compareTo(operateAmount) < 0) {
                throw new ResourceUnavailableException("账户余额不足");
            }
            if (operateType == 0) {
                asset.setTotal(total.add(operateAmount));
            } else if (operateType == 1) {
                asset.setTotal(total.subtract(operateAmount));
            }
        } else if(amountType == 2){
            BigDecimal frozen = asset.getFrozen();
            if (operateType == 1 && frozen.compareTo(operateAmount) < 0) {
                throw new ResourceUnavailableException("账户冻结余额不足");
            }
            if (operateType == 0) {
                asset.setFrozen(frozen.add(operateAmount));
            } else if (operateType == 1) {
                asset.setFrozen(frozen.subtract(operateAmount));
            }
        }
        asset.setUpdateTime(LocalDateTime.now());
        int i = baseMapper.updateById(asset);
        if (i != 1) {
            throw new ResourceUnavailableException("操作失败");
        }
        return asset.getTotal();
    }

    @Override
    public void updatePlatformAsset(BigDecimal platformRealityAmount, BigDecimal payAmount) {
        Asset asset = queryAsset(PlatformConstant.PLATFORM_ID, 0);
        BigDecimal frozen = asset.getFrozen();
        if(frozen.compareTo(payAmount) < 0){
            throw new ResourceUnavailableException("账户冻结余额不足");
        }
        baseMapper.updatePlatformAsset(platformRealityAmount, payAmount, LocalDateTime.now(), PlatformConstant.PLATFORM_ID);
    }

    @Override
    public MerchantAssetDTO getMerchantAsset(Long merchantId) {
        if(merchantId == null){
            return null;
        }
        MerchantAssetDTO dto = new MerchantAssetDTO();
        WalletDTO walletDTO = balanceDetailService.getMerchantAsset(merchantId);
        LambdaQueryWrapper<Asset> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Asset::getUserId, merchantId);
        wrapper.eq(Asset::getType, 2);
        Asset asset = baseMapper.selectOne(wrapper);
        if(asset != null){
            BigDecimal total = asset.getTotal();
            BigDecimal frozen = asset.getFrozen();
            walletDTO.setAsset(total);
            walletDTO.setWithdraw(total.subtract(frozen));
        }
        dto.setWalletDTO(walletDTO);
        OrderDTO orderDTO = orderService.queryOrderManage(merchantId);
        dto.setOrderDTO(orderDTO);
        MerchantTeamDTO merchantTeamDTO = orderService.queryTeamOrder(merchantId);
        dto.setMerchantTeamDTO(merchantTeamDTO);
        return dto;
    }

    @Override
    public void withdraw(WithdrawParam param) {
        Integer type = param.getType();
        BigDecimal amount = param.getAmount();
        Long userId = SessionContextHolder.getId();
        if(type == null || (type != 1 && type != 2)){
            throw new ResourceUnavailableException("提现类型错误");
        }
        if(type == 2){
            //商家体现
            Long merchantId = param.getMerchantId();
            MerchantDTO merchantDTO = merchantService.queryCheck(merchantId, userId);
            if(merchantDTO == null){
                throw new ResourceUnavailableException("该账户下商家不存在");
            }
            userId = merchantId;
        }
        Asset asset = queryAsset(userId, type);
        BigDecimal total = asset.getTotal();
        if(amount.compareTo(total) > 0){
            throw new ResourceUnavailableException("账户余额不足");
        }
        //查询首信易商户号
        MerchantRelation merchantRelation = merchantRelationService.queryByUserIdAndType(userId, type);
        if(merchantRelation == null){
            throw new ResourceUnavailableException("请联系管理员开通商户号");
        }
        Long finalUserId = userId;
        DatabaseContextHolder.transactional((Executor) () -> {
            Long withdrawId = null;
            JSONObject json = null;
            //生成提现订单
            if(type == 2){
                withdrawId = withdrawService.createWithdraw(finalUserId, type, amount, 1, 0);
                //商户提现操作
                String sxyMerchantId = merchantRelation.getSxyMerchantId();
                //生成提现参数
                HashMap<String, Object> map = new HashMap<>();
                map.put("merchantId", sxyMerchantId);
                map.put("requestId", withdrawId);
                map.put("withdrawAmount", amount.multiply(new BigDecimal("100")));
                map.put("notifyUrl", TradeConstant.WITHDRAW_NOTIFY_URL);
                map.put("remark", "");
                Gson gson = new Gson();
                json = TradeHandle.handle(gson.toJson(map),
                        TradeConstant.WITHDRAW_URL, sxyMerchantId, TradeConstant.PARTNER_ID);
            } else {
                //用户先转账后提现
                withdrawId = withdrawService.createWithdraw(finalUserId, 1, amount, 1, 1);
                MerchantRelation relation = merchantRelationService.queryByUserIdAndType(finalUserId, 1);
                //生成转账参数
                HashMap<String, Object> map = new HashMap<>();
                map.put("merchantId", TradeConstant.PARTNER_ID);
                map.put("requestId", withdrawId);
                map.put("receiverId", relation.getSxyMerchantId());
                map.put("receiverType", "MERCHANT_ACC");
                map.put("amount", amount.multiply(new BigDecimal("100")));
                map.put("currency", "CNY");
                map.put("notifyUrl", TradeConstant.TRANSFER_NOTIFY_URL);
                map.put("deputeMark", "NOT_DO_DEPUTE");
                Gson gson = new Gson();
                json = TradeHandle.handle(gson.toJson(map),
                        TradeConstant.TRANSFER_URL, relation.getSxyMerchantId(), TradeConstant.PARTNER_ID);
            }
            String status = json.getString("status");
            if(!"SUCCESS".equals(status)){
                //提现申请失败
                withdrawService.updateStatusById(withdrawId, 2);
                return;
            }
            //冻结金额
            int i = baseMapper.updateFrozenAmount(amount, LocalDateTime.now(), finalUserId, type);
            if(i != 1){
                withdrawService.updateStatusById(withdrawId, 2);
                throw new ResourceUnavailableException("提现申请失败");
            }
        });
    }

    /**
     * 转账
     * @param sourceUserId
     * @param targetUserId
     * @param type
     * @param amount
     */
    @Override
    public void transfer(Long sourceUserId, Long targetUserId, Integer type, BigDecimal amount){
        Long withdrawId = withdrawService.createWithdraw(targetUserId, type, amount, 1, 1);
        MerchantRelation relation = merchantRelationService.queryByUserIdAndType(targetUserId, type);
        //生成转账参数
        HashMap<String, Object> map = new HashMap<>();
        map.put("merchantId", TradeConstant.PARTNER_ID);
        map.put("requestId", withdrawId);
        map.put("receiverId", relation.getSxyMerchantId());
        map.put("receiverType", "MERCHANT_ACC");
        map.put("amount", amount.multiply(new BigDecimal("100")));
        map.put("currency", "CNY");
        map.put("notifyUrl", TradeConstant.TRANSFER_NOTIFY_URL);
        map.put("deputeMark", "NOT_DO_DEPUTE");
        Gson gson = new Gson();
        JSONObject json = TradeHandle.handle(gson.toJson(map),
                TradeConstant.TRANSFER_URL, relation.getSxyMerchantId(), TradeConstant.PARTNER_ID);
        String status = json.getString("status");
        if(!"SUCCESS".equals(status)){
            //转账申请失败
            withdrawService.updateStatusById(withdrawId, 2);
            return;
        }
        //冻结金额
        int i = baseMapper.updateFrozenAmount(amount, LocalDateTime.now(), sourceUserId, type);
        if(i != 1){
            withdrawService.updateStatusById(withdrawId, 2);
            throw new ResourceUnavailableException("提现申请失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void withdrawNotify(WithdrawNotifyParam param) {
        String requestId = param.getRequestId();
        Long id = Long.parseLong(requestId);
        OmsWithdraw withdraw = withdrawService.queryById(id);
        if(withdraw == null){
            return;
        }
        String withdrawAmount = param.getWithdrawAmount();
        BigDecimal amount = withdraw.getAmount();
        if(amount.compareTo(new BigDecimal(withdrawAmount)) != 0){
            //金额不一致
            return;
        }
        String withdrawStatus = param.getWithdrawStatus();
        int status = 2;
        if("SUCCESS".equals(withdrawStatus)){
            status = 1;
        }
        //更新提现订单状态和外部订单号
        int i = withdrawService.updateByNotify(id, param.getOrderId(), status);
        if(i == 1){
            if(status == 2){
                //失败解冻金额
                baseMapper.backFrozen(amount, withdraw.getUserId(), withdraw.getType(), LocalDateTime.now());
            } else {
                baseMapper.deductFrozen(amount, withdraw.getUserId(),withdraw.getType(), LocalDateTime.now());
            }
        }
    }

    @Override
    public void transferNotify(TransferNotifyParam param) {
        String requestId = param.getRequestId();
        Long id = Long.parseLong(requestId);
        OmsWithdraw withdraw = withdrawService.queryById(id);
        if(withdraw == null){
            return;
        }
        String withdrawAmount = param.getAmount();
        BigDecimal amount = withdraw.getAmount();
        if(amount.compareTo(new BigDecimal(withdrawAmount)) != 0){
            //金额不一致
            return;
        }
        String transferStatus = param.getStatus();
        int status = 2;
        if("SUCCESS".equals(transferStatus)){
            status = 1;
        }
        //更新转账订单状态和外部订单号
        int i = withdrawService.updateByNotify(id, null, status);
        if(i == 1){
            if(status == 2){
                //失败解冻金额
                baseMapper.backFrozen(amount, withdraw.getUserId(), withdraw.getType(), LocalDateTime.now());
            } else {
                Integer type = withdraw.getType();
                if(type == 2){
                    //商家转账-更新余额
                    baseMapper.deductFrozen(amount, withdraw.getUserId(), withdraw.getType(), LocalDateTime.now());
                } else {
                    //用户转账-继续提现操作
                    //用户提现
                    Long withdrawId = withdrawService.createWithdraw(withdraw.getUserId(), 1, amount, 1, 0);
                    //商户提现操作
                    String sxyMerchantId = param.getReceiverId();
                    //生成提现参数
                    HashMap<String, Object> map = new HashMap<>();
                    map.put("merchantId", sxyMerchantId);
                    map.put("requestId", withdrawId);
                    map.put("withdrawAmount", amount.multiply(new BigDecimal("100")));
                    map.put("notifyUrl", TradeConstant.WITHDRAW_NOTIFY_URL);
                    map.put("remark", "");
                    Gson gson = new Gson();
                    JSONObject json = TradeHandle.handle(gson.toJson(map),
                            TradeConstant.WITHDRAW_URL, sxyMerchantId, TradeConstant.PARTNER_ID);
                    String result = json.getString("status");
                    if(!"SUCCESS".equals(result)){
                        //提现申请失败
                        withdrawService.updateStatusById(withdrawId, 2);
                    }
                }
            }
        }
    }
}
