package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.FavoriteType;
import com.chelvc.cloud.vehicle.client.dto.FavoriteDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.param.FavoritePagingParam;
import com.chelvc.cloud.vehicle.client.param.FavoriteQueryParam;
import com.chelvc.cloud.vehicle.server.copier.FavoriteCopier;
import com.chelvc.cloud.vehicle.server.copier.GoodsCopier;
import com.chelvc.cloud.vehicle.server.dao.FavoriteMapper;
import com.chelvc.cloud.vehicle.server.entity.Favorite;
import com.chelvc.cloud.vehicle.server.service.FavoriteService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 用户收藏业务操作实现
 *
 * @author liude
 * @date 2023/7/20
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FavoriteServiceImpl extends ServiceImpl<FavoriteMapper, Favorite> implements FavoriteService {

    private final GoodsService goodsService;

    @Override
    public Long addMerchantFavorite(@NonNull Long merchantId, String type) {
        Long userId = SessionContextHolder.getId();
        if ("GOODS".equals(type)) {
            Integer count = this.lambdaQuery()
                    .eq(Favorite::getUserId, userId)
                    .eq(Favorite::getType, FavoriteType.GOODS)
                    .eq(Favorite::getContentId, merchantId).count();
            AssertUtils.available(count <= 0, "收藏信息已存在");
            Favorite favorite = Favorite.builder().userId(userId).type(FavoriteType.GOODS)
                    .contentId(merchantId).createTime(new Date()).build();
            this.baseMapper.saveFavorite(favorite);
            return favorite.getId();
        } else {
            Integer count = this.lambdaQuery()
                    .eq(Favorite::getUserId, userId)
                    .eq(Favorite::getType, FavoriteType.MERCHANT)
                    .eq(Favorite::getContentId, merchantId).count();
            AssertUtils.available(count <= 0, "收藏信息已存在");
            Favorite favorite = Favorite.builder().userId(userId).type(FavoriteType.MERCHANT)
                    .contentId(merchantId).createTime(new Date()).build();
            this.baseMapper.saveFavorite(favorite);
            return favorite.getId();
        }
    }

    @Override
    public void deleteMineFavorite(@NonNull Long id) {
        Long userId = SessionContextHolder.getId();
        Favorite favorite = this.lambdaQuery()
                .eq(Favorite::getUserId, userId)
                .eq(Favorite::getContentId, id).one();
        AssertUtils.available(favorite != null, "收藏信息不存在");
        AssertUtils.nonnull(favorite.getUserId(), "非法操作");
        AssertUtils.available(Objects.equals(favorite.getUserId(), SessionContextHolder.getId()), "非法操作");
        this.baseMapper.deleteFavorite(id, userId);
    }

    @Override
    public Pagination<MerchantDTO> getFavoriteMerchantPaging(FavoritePagingParam param) {
        return null;
    }

    @Override
    public List<FavoriteDTO> listFavoriteMerchants(@NonNull FavoriteQueryParam param) {
        Long userId = SessionContextHolder.getId();
        List<Favorite> favorites = this.baseMapper.listFavoriteMerchants(userId, param);
        if (CollectionUtils.isEmpty(favorites)) {
            return Collections.emptyList();
        }
        return favorites
                .stream()
                .map(o -> {
                    FavoriteDTO favoriteDTO = FavoriteCopier.INSTANCE.copying(o);
                    favoriteDTO.setMerchant(o.getMerchant());
                    return favoriteDTO;
                }).collect(Collectors.toList());

    }

    @Override
    public List<FavoriteDTO> listFavoriteGoods() {
        Long userId = SessionContextHolder.getId();
        List<Favorite> favorites = this.lambdaQuery().eq(Favorite::getUserId, userId)
                .eq(Favorite::getType, FavoriteType.GOODS).list();

        List<FavoriteDTO> favoriteList = FavoriteCopier.INSTANCE.copying(favorites);
        favoriteList.forEach(item -> {
            item.setGoods(GoodsCopier.INSTANCE.copying(this.goodsService.getById(item.getContentId())));
        });
        return favoriteList;
    }
}
