package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.user.client.model.Scope;
import com.chelvc.cloud.vehicle.client.constant.ReserveStatus;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyIdsDTO;
import com.chelvc.cloud.vehicle.client.dto.OrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyPagingParam;
import com.chelvc.cloud.vehicle.server.copier.OrderReturnApplyCopier;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderReturnApplyMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderReturnApply;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderItemService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderReturnApplyService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderService;
import com.chelvc.cloud.vehicle.server.service.ReservationService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 订单退货申请Service业务层处理
 *
 * @author liude
 * @date 2023-11-08
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderReturnApplyServiceImpl extends ServiceImpl<OmsOrderReturnApplyMapper, OmsOrderReturnApply> implements
        OmsOrderReturnApplyService
{
    private final MerchantService merchantService;

    private final OmsOrderService omsOrderService;

    private final OmsOrderItemService omsOrderItemService;

    private final UserClient userClient;

    private final ReservationService reservationService;

    /**
     * 查询订单退货申请
     *
     * @param id 订单退货申请主键
     * @return 订单退货申请
     */
    @Override
    public OrderReturnApplyDTO selectOmsOrderReturnApplyById(@NonNull Long id)
    {
        OmsOrderReturnApply omsOrderReturnApply = this.getById(id);
        OrderReturnApplyDTO orderReturnApplyDTO = OrderReturnApplyCopier.INSTANCE.copying(omsOrderReturnApply);
        orderReturnApplyDTO.setGoodsList(this.omsOrderItemService.getOrderItemByOrderId(orderReturnApplyDTO.getOrderId()));
        return orderReturnApplyDTO;
    }

    /**
     * 查询订单退货申请列表
     *
     * @param param 订单退货申请
     * @return 订单退货申请
     */
    @Override
    public List<OrderReturnApplyDTO> selectOmsOrderReturnApplyList(OrderReturnApplyModifyParam param)
    {
        List<OmsOrderReturnApply> list = this.lambdaQuery().eq(OmsOrderReturnApply::getOrderId,param.getOrderId())
                .orderByDesc(OmsOrderReturnApply::getCreateTime).list();
        return OrderReturnApplyCopier.INSTANCE.copying(list);
    }

    /**
     * 新增订单退货申请
     *
     * @param param 订单退货申请
     * @return 结果
     */
    @Override
    public Long insertOmsOrderReturnApply(@NonNull OrderReturnApplyModifyParam param)
    {
        OmsOrderReturnApply omsOrderReturnApply = OrderReturnApplyCopier.INSTANCE.copying(param);
        omsOrderReturnApply.setCreateTime(new Date());
        omsOrderReturnApply.setHandleTime(new Date());
        OmsOrder omsOrder = this.omsOrderService.getById(param.getOrderId());
        if (null != omsOrder){
            omsOrderReturnApply.setReturnAmount(omsOrder.getPayAmount());
            UserDTO userDTO = this.userClient.getUser(omsOrder.getUserId());
            omsOrderReturnApply.setReturnName(userDTO.getNickname());
            omsOrderReturnApply.setReturnPhone(userDTO.getMobile());
            omsOrderReturnApply.setStatus(0L);

        }
        this.save(omsOrderReturnApply);
        this.omsOrderService.updateOrderStatus(param.getOrderId(),5);
        this.reservationService.updateStatusByOrderId(param.getOrderId(), ReserveStatus.CANCELED);
        return omsOrderReturnApply.getId();
    }

    /**
     * 修改订单退货申请
     *
     * @param param 订单退货申请
     * @return 结果
     */
    @Override
    public Long updateOmsOrderReturnApply(@NonNull Long id, @NonNull OrderReturnApplyModifyParam param)
    {
        OmsOrderReturnApply omsOrderReturnApply = AssertUtils.nonnull(this.getById(id), "退货申请单不存在");
        OrderReturnApplyCopier.INSTANCE.copying(param,omsOrderReturnApply);
        return omsOrderReturnApply.getId();
    }

    /**
     * 批量删除订单退货申请
     *
     * @param ids 需要删除的订单退货申请主键
     * @return 结果
     */
    @Override
    public void deleteOmsOrderReturnApplyByIds(@NonNull List<Long> ids)
    {
        this.baseMapper.deleteBatchIds(ids);
    }

    /**
     * 删除订单退货申请信息
     *
     * @param id 订单退货申请主键
     * @return 结果
     */
    @Override
    public Long deleteOmsOrderReturnApplyById(@NonNull Long id)
    {
        OmsOrderReturnApply omsOrderReturnApply = AssertUtils.nonnull(this.getById(id), "退货申请单不存在");
        this.baseMapper.deleteById(id);
        return omsOrderReturnApply.getId();
    }
    @Override
    public Pagination<OrderReturnApplyDTO> getOrderReturnApplyPaging(OrderReturnApplyPagingParam param){
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            param.setMerchantId(merchants.get(0));
        }
        Page<OmsOrderReturnApply> page = this.lambdaQuery()
                .eq(StringUtils.notEmpty(param.getMerchantId()),OmsOrderReturnApply::getMerchantId,param.getMerchantId())
                .eq(StringUtils.notEmpty(param.getOrderId()),OmsOrderReturnApply::getOrderId,param.getOrderId())
                .orderByDesc(OmsOrderReturnApply::getCreateTime)
                .orderByDesc(OmsOrderReturnApply::getReceiveTime)
                .page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<OrderReturnApplyDTO> pages = DatabaseContextHolder.pagination(page, OrderReturnApplyCopier.INSTANCE::copying);
        if (CollectionUtils.isEmpty(pages.getRecords())){
            return pages;
        }
        pages.getRecords().forEach(item ->{
            item.setGoodsList(this.omsOrderItemService.getOrderItemByOrderId(item.getOrderId()));

        });
        return pages;
    }

    @Override
    public Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(Long merchantId, Paging paging) {
        Page<OmsOrderReturnApply> page = this.lambdaQuery()
                .eq(OmsOrderReturnApply::getMerchantId, merchantId)
                .orderByAsc(OmsOrderReturnApply::getCreateTime).page(DatabaseContextHolder.page(paging));
        List<OmsOrderReturnApply> omsOrderReturnApplies = page.getRecords();
        List<OmsOrderReturnApplyIdsDTO> copy = ObjectUtils.copying(omsOrderReturnApplies, OmsOrderReturnApplyIdsDTO.class);
        return DatabaseContextHolder.pagination(page, copy);
    }

    @Override
    public Integer queryReturnOrderNum(Long merchantId) {
        LambdaQueryWrapper<OmsOrderReturnApply> lwq = Wrappers.lambdaQuery();
        lwq.eq(OmsOrderReturnApply::getMerchantId, merchantId);
        lwq.eq(OmsOrderReturnApply::getStatus, 0);
        return baseMapper.selectCount(lwq);
    }

    @Override
    public OmsOrderReturnApplyDTO getOrderReturnApplyInfo(Long id) {
        OmsOrderReturnApply omsOrderReturnApply = this.getById(id);
        if (omsOrderReturnApply == null){
            return null;
        }
        return ObjectUtils.copying(omsOrderReturnApply, OmsOrderReturnApplyDTO.class);
    }
}
